@@ -1,7 +1,7 @@
 #!/usr/bin/env perl
-# $Id: tlmgrgui.pl 30372 2013-05-10 17:49:37Z karl $
+# $Id: tlmgrgui.pl 32836 2014-02-01 03:11:02Z preining $
 #
-# Copyright 2009-2013 Norbert Preining
+# Copyright 2009-2014 Norbert Preining
 # This file is licensed under the GNU General Public License version 2
 # or any later version.
 #
@@ -15,8 +15,8 @@
 $^W = 1;
 use strict;
 
-my $guisvnrev = '$Revision: 30372 $';
-my $guidatrev = '$Date: 2013-05-10 19:49:37 +0200 (Fri, 10 May 2013) $';
+my $guisvnrev = '$Revision: 32836 $';
+my $guidatrev = '$Date: 2014-02-01 04:11:02 +0100 (Sat, 01 Feb 2014) $';
 my $tlmgrguirevision;
 if ($guisvnrev =~ m/: ([0-9]+) /) {
   $tlmgrguirevision = $1;
@@ -110,6 +110,8 @@ my $darktext;
 my $match_entry;
 my $loaded_text;
 my $loaded_text_button;
+my $default_repo;
+my $update_all_button;
 
 my %settings_label;
 
@@ -168,7 +170,10 @@ sub guimain {
   build_initial_gui();
   init_hooks();
 
-  info(__("Loading local TeX Live database;\nthis may take some time, please be patient ...") . "\n");
+  info(__("Loading local TeX Live database") . 
+       "\n  ($::maintree/$InfraLocation/$DatabaseName)\n" . 
+       __("This may take some time, please be patient ...") . 
+       "\n");
 
   # call the init function from tlmgr.pl
   # with 0 as argument, so that it does not call die on errors.
@@ -280,17 +285,26 @@ sub do_rest_of_gui {
         -separator => "/",
         -selectmode => "none");
 
-  my $button_frame = $mw->Frame;
+  my $button_frame = $mw->Labelframe(-text => __("Repository"));
+  $loaded_text = $button_frame->Label(-text => __("Loaded:") . " " . __("none"));
+  $loaded_text->pack(@left, @p_ii);
+
   $loaded_text_button = $button_frame->Button(-text => __("Load default"),
     -command => sub { setup_location($tlpdb_location); });
-  $loaded_text_button->pack(
-      -anchor => 'e', -padx => 0, -pady => 0,
-      -ipadx => 0, -ipady => 0, -side => 'right');
-  $loaded_text = $button_frame->Label(
-    -text => __("Loaded repository:") . " " . __("none"));
-  $loaded_text->pack(-anchor => 'e', -padx => 0, -pady => 0, 
-    -ipadx => 0, -ipady => 0, -side => 'right');
-    
+  $loaded_text_button->pack( @left, @p_ii);
+
+  my %repos = repository_to_array($tlpdb_location);
+  my @tags = keys %repos;
+  my $foo;
+  if ($#tags > 0) {
+    $foo = __("multiple repositories");
+  } else {
+    $foo = $tlpdb_location;
+  }
+
+  $default_repo = $button_frame->Label(-text => __("Default:") . " " . $foo );
+  $default_repo->pack(@left, @p_ii);
+
   #$button_frame->pack(-expand => 1, -fill => 'x', @p_ii);
 
   #my $top_frame = $mw->Labelframe(-text => __("Display configuration"));
@@ -414,10 +428,11 @@ $g->headerCreate(4, @htype, -itemtype => 'text', -text => __("Short description"
 
   my $with_all_frame = $actions_frame->Frame;
   $with_all_frame->pack(@left, -padx => '5m');
-  $with_all_frame->Button(-text => __('Update all installed'),
-                          -state => $::action_button_state,
-                          -command => sub { update_all_packages(); }
-    )->pack(@p_ii);
+  $update_all_button =
+    $with_all_frame->Button(-text => __('Update all installed'),
+                            -state => $::action_button_state,
+                            -command => sub { update_all_packages(); }
+      )->pack(@p_ii);
   $with_all_frame->Checkbutton(-text => __("Reinstall previously removed packages"), 
     -variable => \$opts{"reinstall-forcibly-removed"})->pack();
 
@@ -511,13 +526,13 @@ sub setup_menu_system {
     $tlpdb_location_string = __("multiple repositories");
   }
   $menu_file->add('command', 
-    -label => __("Load default repository:") . " $tlpdb_location_string",
+    -label => __("Load default (from tlpdb) repository:") . " $tlpdb_location_string",
     -command => sub { setup_location($tlpdb_location); });
   if (defined($cmdline_location)) {
     $menu_file->add('command', -label => __("Load cmd line repository:") . " $cmdline_location",
       -command => sub { setup_location($cmdline_location); });
   }
-  $menu_file->add('command', -label => __("Load default net repository:") . " $TeXLiveURL",
+  $menu_file->add('command', -label => __("Load standard net repository:") . " $TeXLiveURL",
     -command => sub { setup_location($TeXLiveURL); });
   if ($mode_expert) {
     $menu_file->add('command', -label => __("Load other repository ..."),
@@ -638,7 +653,7 @@ sub setup_menu_system {
 
 tlmgrgui revision $tlmgrguirevision
 $tlmgrrev
-Copyright 2009-2012 Norbert Preining
+Copyright 2009-2014 Norbert Preining
 
 Licensed under the GNU General Public License version 2 or higher
 In case of problems, please contact: texlive\@tug.org"
@@ -846,6 +861,7 @@ sub update_grid {
     $crit_match = 1;
     $lighttext->configure(-foreground => 'red');
     $darktext->configure(-foreground => 'red');
+    $update_all_button->configure(-text => __('Update the TeX Live Manager'));
   } else {
     @displist = (@schemes, @colls, @packs);
   }
@@ -1894,7 +1910,8 @@ sub install_selected_packages {
   my @foo = SelectedPackages();
   if (@foo) {
     # that doesn't hurt if it is already loaded
-    init_install_media();
+    # it does hurt when there are critical updates ... so don't do it
+    #init_install_media();
     my @args = qw/install/;
     push @args, @foo;
     execute_action_gui(@args);
@@ -1924,6 +1941,21 @@ sub SelectedPackages {
   return @ret;
 }
 
+sub critical_updates_done_msg_and_end {
+  # terminate here immediately so that we are sure the auto-updater
+  # is run immediately
+  # make sure we exit in finish(0)
+  $::gui_mode = 0;
+  # warn that program will now be terminated
+  $mw->Dialog(-title => __("Warning"),
+    -text => __("Critical updates have been installed.\nProgram will terminate now.\nPlease restart if necessary."),
+    -buttons => [ __("Ok") ])->Show;
+  # also delete the main window before we kill the process to 
+  # make sure that Tk is happy (segfault on cmd line, email Taco)
+  $mw->destroy;
+  finish(0); 
+}
+  
 sub update_all_packages {
   my @args = qw/update/;
   if (@critical_updates) {
@@ -1932,21 +1964,11 @@ sub update_all_packages {
     $opts{"all"} = 1;
   }
   # that doesn't hurt if it is already loaded
-  init_install_media();
+  # it does hurt when there are critical updates ... so don't do it
+  #init_install_media();
   execute_action_gui(qw/update/);
   if (@critical_updates) {
-    # terminate here immediately so that we are sure the auto-updater
-    # is run immediately
-    # make sure we exit in finish(0)
-    $::gui_mode = 0;
-    # warn that program will now be terminated
-    $mw->Dialog(-title => __("Warning"),
-      -text => __("Critical updates have been installed.\nProgram will terminate now.\nPlease restart if necessary."),
-      -buttons => [ __("Ok") ])->Show;
-    # also delete the main window before we kill the process to 
-    # make sure that Tk is happy (segfault on cmd line, email Taco)
-    $mw->destroy;
-    finish(0); 
+    critical_updates_done_msg_and_end();
   }
   reinit_local_tlpdb();
 }
@@ -1955,7 +1977,8 @@ sub update_selected_packages {
   my @foo = SelectedPackages();
   if (@foo) {
     # that doesn't hurt if it is already loaded
-    init_install_media();
+    # it does hurt when there are critical updates ... so don't do it
+    #init_install_media();
     my @args = qw/update/;
     # argument processing
     # in case we have critical updates present we do put the list of
@@ -1966,18 +1989,7 @@ sub update_selected_packages {
     push @args, @foo;
     execute_action_gui(@args);
     if (@critical_updates) {
-      # terminate here immediately so that we are sure the auto-updater
-      # is run immediately
-      # make sure we exit in finish(0)
-      $::gui_mode = 0;
-      # warn that program will now be terminated
-      $mw->Dialog(-title => __("Warning"),
-        -text => __("Critical updates have been installed.\nProgram will terminate now.\nPlease restart if necessary."),
-        -buttons => [ __("Ok") ])->Show;
-      # also delete the main window before we kill the process to 
-      # make sure that Tk is happy (segfault on cmd line, email Taco)
-      $mw->destroy;
-      finish(0); 
+      critical_updates_done_msg_and_end();
     }
     reinit_local_tlpdb();
   }
@@ -2282,9 +2294,16 @@ sub update_list_remote {
     # create the warning dialog
     #
     my $sw = $mw->DialogBox(-title => __("Warning"), -buttons => [ __("Ok") ]);
-    my $t = __("Updates for the tlmgr are present.\nInstallation and upgrades won't work without being forced.\nPlease select \"Update all installed\" button after dismissing this dialogue.\n\nThe program will terminate after the update.\nThen you can restart the program for further updates.");
+    my $t = __("The TeX Live manager (the software you're currently running)
+needs to be updated before any other updates can be done.
+
+Please do this by clicking the \"Update the TeX Live Manager\" button,
+after dismissing this dialogue.
+
+After the update, the TeX Live manager will terminate.
+You can then restart it to proceed with further updates."); 
     if ($min_action) {
-      $t .= "\n\n" . __("At least %s further updates are available after tlmgr has been updated.", $min_action);
+      $t .= "\n\n" . __("(At least %s further updates are available after tlmgr has been updated.)", $min_action);
     }
     $t .= "\n\n" . __("Please wait a bit after the program has terminated so that the update can be completed.") if win32();
     $sw->add("Label", -text => $t)->pack(-padx => "3m", -pady => "3m");
@@ -2526,11 +2545,12 @@ sub cb_edit_string_or_dir {
 
 sub cb_edit_location {
   my $key = shift;
+  my $okbutton;
   my $val;
   my $sw = $mw->Toplevel(-title => __("Load package repository"));
   $sw->transient($mw);
   $sw->grab();
-  $sw->Label(-text => __("Package repository:"))->pack(@p_ii);
+  $sw->Label(-text => __("Load this package repository:"))->pack(@p_ii);
   my @mirror_list;
   push @mirror_list, TeXLive::TLUtils::create_mirror_list();
   my $entry = $sw->BrowseEntry(
@@ -2543,33 +2563,51 @@ sub cb_edit_location {
       sub {
         if ($val !~ m/^  /) {
           $val = "";
+          $okbutton->configure(-state => 'disabled');
         } elsif ($val =~ m!(http|ftp)://!) {
           $val = TeXLive::TLUtils::extract_mirror_entry($val);
+          $okbutton->configure(-state => 'normal');
         } else {
           $val =~ s/^\s*//;
+          $okbutton->configure(-state => 'normal');
         }
       },
     -variable => \$val);
   # end new
   $entry->pack(@p_ii);
   my $f1 = $sw->Frame;
-  $f1->Button(-text => __("Choose Directory"),
+  $f1->Button(-text => __("Choose local directory"),
     -command => sub {
                       my $var = $sw->chooseDirectory();
                       if (defined($var)) {
                         $val = $var;
+                        $okbutton->configure(-state => 'normal');
                       }
                     })->pack(@left, @p_ii);
-  $f1->Button(-text => __("Default net package repository"),
+  $f1->Button(-text => __("Use standard net repository"),
     -command => sub {
                       $val = $TeXLiveURL;
+                      $okbutton->configure(-state => 'normal');
                     })->pack(@left, @p_ii);
   $f1->pack;
   my $f = $sw->Frame;
-  my $okbutton = $f->Button(-text => __("Ok"),
-    -command => sub { $location = $val;
-                      $sw->destroy;
-                      setup_location($location);
+  $okbutton = $f->Button(-text => __("Load"), -state => "disabled",
+    -command => sub { 
+                      if ($val) {
+                        $location = $val;
+                        $sw->destroy;
+                        my $foo = $mw->Toplevel();
+                        $foo->transient($mw);
+                        $foo->overrideredirect(1);
+                        my $frame = $foo->Frame( -border => 5, -relief => 'groove' )->pack;
+                        $frame->Label( -text => __("Loading remote repository - this may take some time, please be patient ...") )->pack( -padx => 5 );
+                        $foo->Popup(-popanchor => 'c');
+                        setup_location($location);
+                        $foo->destroy;
+                      } else {
+                        # button should be disabled and not clickable 
+                        # why are we here???
+                      }
                     })->pack(@left, @p_ii);
   my $cancelbutton = $f->Button(-text => __("Cancel"),
           -command => sub { $sw->destroy })->pack(@right, @p_ii);
@@ -2601,9 +2639,10 @@ sub update_loaded_location_string {
   } else {
     $loaded_text_button->configure(-text => $arg);
   }
-  $loaded_text->configure(-text => __("Loaded repository:"));
+  $loaded_text->configure(-text => __("Loaded:"));
   $loaded_text_button->configure( -command => 
       sub { transient_show_multiple_repos($loaded_text_button, @vals); });
+  $default_repo->packForget;
 }
 
 sub transient_show_multiple_repos {
@@ -2674,7 +2713,7 @@ on the command line.
   } else {
     my $io = IO::String->new($txt);
     my $parser = Pod::Text->new (sentence => 0, width => 78);
-    $parser->parse_from_file("$Master/texmf/scripts/texlive/tlmgr.pl", $io);
+    $parser->parse_from_file("$Master/texmf-dist/scripts/texlive/tlmgr.pl", $io);
   }
   my $sw = $mw->Toplevel(-title => __("Help"));
   $sw->transient($mw);
