@@ -178,6 +178,8 @@ authors = {
 	"rene_bastian"        => ["René", "Bastian", nil, false, false], # TODO make contact
 	"daniel_flipo"        => ["Daniel", "Flipo", nil, false, false], # TODO make contact
 	"bernard_gaulle"      => ["Bernard", "Gaulle", nil, false, false], # Deceased...
+	"theppitak_karoonboonyanan" => ["Theppitak", "Karoonboonyanan" "thep{at}linux{dot}thai{dot}net", true, true],
+	"levan_shoshiashvili" => ["Levan", "Shoshiashvili", "shoshia{at}hotmail{dot}com", true, true],
 }
 
 
@@ -411,8 +413,8 @@ class Languages < Hash
 	"exceptions" => false,
 	"message" => "German hyphenation patterns (traditional orthography)",
 
-	"version"       => nil,
-	"last_modified" => "2012-05-30", # For the active project
+	"version"       => "0.3",
+	"last_modified" => "2013-05-26", # For the active project
 	"type"          => "dictionary",
 	"authors"       => ["werner_lemberg"],
 	"licence"       => "LPPL",
@@ -450,8 +452,8 @@ class Languages < Hash
 	"exceptions" => false,
 	"message" => "German hyphenation patterns (reformed orthography)",
 
-	"version"       => nil,
-	"last_modified" => "2012-05-30", # For the active project
+	"version"       => "0.3",
+	"last_modified" => "2013-05-26", # For the active project
 	"type"          => "dictionary",
 	"authors"       => ["werner_lemberg"],
 	"licence"       => "LPPL",
@@ -472,8 +474,8 @@ class Languages < Hash
 	"exceptions" => false,
 	"message" => "Swiss-German hyphenation patterns (traditional orthography)",
 
-	"version"       => nil,
-	"last_modified" => "2012-05-30", # For the active project
+	"version"       => "0.3",
+	"last_modified" => "2013-05-26", # For the active project
 	"type"          => "dictionary",
 	"authors"       => ["werner_lemberg"],
 	"licence"       => "LPPL",
@@ -1131,6 +1133,30 @@ class Languages < Hash
 		"dated November 6, 1996 and its following amendments.",
 	],
 },
+# piedmontese
+{
+	"code" => "pms",
+	"name" => "piedmontese",
+	"use_new_loader" => true,
+	"use_old_patterns" => false,
+	"filename_old_patterns" => nil,
+	"hyphenmin" => [2,2],
+	"encoding" => "ascii",
+	"exceptions" => false,
+	"message" => "Piedmontese hyphenation patterns",
+
+	"version"       => "1.0",
+	"last_modified" => "2013-02-14",
+	"type"          => "rules",
+	"authors"       => ["claudio_beccari"],
+	"licence"       => "LPPL", # Status: maintained!
+	"description_s" => "Piedmontese hyphenation patterns",
+	"description_l" => [
+		#......................................................................#
+		"Hyphenation patterns for Piedmontese in ASCII encoding.",
+		"Compliant with 'Gramatica dla lengua piemonteisa' by Camillo Brero.",
+	],
+},
 # kurmanji
 {
 	"code" => "kmr",
@@ -1673,7 +1699,7 @@ class Languages < Hash
 	"name" => "sanskrit",
 	"use_new_loader" => true,
 	"use_old_patterns" => false,
-	"hyphenmin" => [1,5], # polyglossia
+	"hyphenmin" => [1,3], # polyglossia
 	"encoding" => nil, # no patterns for 8-bit engines
 	"exceptions" => false,
 	"message" => "Sanskrit hyphenation patterns (v0.6, 2011/09/14)",
@@ -1872,6 +1898,28 @@ class Languages < Hash
 	"exceptions" => false,
 	"message" => "Telugu hyphenation patterns",
 },
+# thai
+{
+	"code" => "th",
+	"name" => "thai",
+	"use_new_loader" => true,
+	"use_old_patterns" => false,
+	"hyphenmin" => [2,3],
+	"encoding" => "lth",
+	"exceptions" => false,
+	"message" => "Thai hyphenation patterns",
+
+	"version"       => nil,
+	"last_modified" => "2013-01-28",
+	"type"          => "dictionary",
+	"authors"       => [ "theppitak_karoonboonyanan" ],
+	"licence"       => "LPPL",
+	"description_s" => "Thai hyphenation patterns",
+	"description_l" => [
+		#......................................................................#
+		"Hyphenation patterns for Thai in LTH and UTF-8 encodings.",
+	],
+},
 # lao
 #{
 #	"code" => "lo",
@@ -1918,6 +1966,28 @@ class Languages < Hash
 		"by files tailored to individual languages.",
 	],
 },
+# georgian
+#{
+#	"code" => "ka",
+#	"name" => "georgian",
+#	"use_new_loader" => true,
+#	"use_old_patterns" => false,
+#	"hyphenmin" => [2,3], # TODO
+#	"encoding" => "t8m",
+#	"exceptions" => false,
+#	"message" => "Georgian hyphenation patterns",
+#
+#	"version"       => nil,
+#	"last_modified" => "2013-02-25", # TODO
+#	"type"          => "dictionary",
+#	"authors"       => [ "levan_shoshiashvili" ],
+#	"licence"       => "other-free",
+#	"description_s" => "Georgian hyphenation patterns",
+#	"description_l" => [
+#		#......................................................................#
+#		"Hyphenation patterns for Georgian in T8M, T8K and UTF-8 encodings.",
+#	],
+#},
 # dumylang -> dumyhyph.tex
 # nohyphenation -> zerohyph.tex
 # arabic -> zerohyph.tex
