@@ -1,5 +1,5 @@
 if not modules then modules = { } end modules ["features"] = {
-    version   = "2.3a",
+    version   = "2.4",
     comment   = "companion to luaotfload.lua",
     author    = "Hans Hagen, Khaled Hosny, Elie Roux, Philipp Gesang",
     copyright = "PRAGMA ADE / ConTeXt Development Team",
@@ -14,11 +14,25 @@ local lpegmatch         = lpeg.match
 ---[[ begin included font-ltx.lua ]]
 --- this appears to be based in part on luatex-fonts-def.lua
 
-local fonts = fonts
+local fonts             = fonts
+local definers          = fonts.definers
+local handlers          = fonts.handlers
+
+local as_script, normalize
+
+if handlers then
+    normalize = handlers.otf.features.normalize
+else
+    normalize = function () end
+    as_script = true
+end
+
 
 --HH A bit of tuning for definitions.
 
-fonts.constructors.namemode = "specification" -- somehow latex needs this (changed name!) => will change into an overload
+if fonts.constructors then
+    fonts.constructors.namemode = "specification" -- somehow latex needs this (changed name!) => will change into an overload
+end
 
 --[[HH--
     tricky: we sort of bypass the parser and directly feed all into
@@ -39,9 +53,6 @@ local stringformat     = string.format
 local stringis_empty   = string.is_empty
 local mathceil         = math.ceil
 
---- TODO an option to dump the default features for a script would make
----      a nice addition to luaotfload-tool
-
 local defaults = {
     dflt = {
         "ccmp", "locl", "rlig", "liga", "clig",
@@ -778,7 +789,7 @@ local set_default_features = function (speclist)
         if support_incomplete[script] then
             report("log", 0, "load",
                 "support for the requested script: "
-                .. "“%s” may be incomplete", script)
+                .. "%q may be incomplete", script)
         end
     else
         script = "dflt"
@@ -792,7 +803,7 @@ local set_default_features = function (speclist)
     local requested = defaults[script]
     if not requested then
         report("log", 0, "load",
-            "no defaults for script “%s”, falling back to “dflt”",
+            "no defaults for script %q, falling back to \"dflt\"",
             script)
         requested = defaults.dflt
     end
@@ -924,7 +935,7 @@ end
 --doc]]--
 
 local handle_invalid_option = function (opt)
-    report("log", 0, "load", "font option “%s” unknown.", opt)
+    report("log", 0, "load", "font option %q unknown.", opt)
     return "", false
 end
 
@@ -942,7 +953,7 @@ local check_garbage = function (_,i, garbage)
             "warning: path in file: lookups is deprecated; ")
         report("log", 0, "load", "use bracket syntax instead!")
         report("log", 0, "load",
-            "position: %d; full match: “%s”",
+            "position: %d; full match: %q",
             i, garbage)
         return true
     end
@@ -1086,9 +1097,9 @@ local select_lookup = function (request)
 end
 
 local supported = {
-    b    = "bold",
-    i    = "italic",
-    bi   = "bolditalic",
+    b    = "b",
+    i    = "i",
+    bi   = "bi",
     aat  = false,
     icu  = false,
     gr   = false,
@@ -1134,7 +1145,7 @@ local handle_request = function (specification)
         --- in an anonymous lookup;
         --- we try to behave as friendly as possible
         --- just go with it ...
-        report("log", 1, "load", "invalid request “%s” of type anon",
+        report("log", 1, "load", "invalid request %q of type anon",
             specification.specification)
         report("log", 1, "load",
                "use square bracket syntax or consult the documentation.")
@@ -1185,20 +1196,21 @@ local handle_request = function (specification)
     --- The next line sets the “rand” feature to “random”; I haven’t
     --- investigated it any further (luatex-fonts-ext), so it will
     --- just stay here.
-    specification.features.normal
-        = fonts.handlers.otf.features.normalize(request.features)
+    specification.features.normal = normalize (request.features)
     return specification
 end
 
-local compare_requests = function (spec)
-    local old = old_behavior(spec)
-    local new = handle_request(spec)
-    return new
+if as_script == true then --- skip the remainder of the file
+    fonts.names.handle_request = handle_request
+    report ("log", 5, "load",
+            "Exiting early from luaotfload-features.lua.")
+    return
+else
+    local registersplit = definers.registersplit
+    registersplit (":", handle_request, "cryptic")
+    registersplit ("",  handle_request, "more cryptic") -- catches \font\text=[names]
 end
 
-fonts.definers.registersplit(":", handle_request, "cryptic")
-fonts.definers.registersplit("",  handle_request, "more cryptic") -- catches \font\text=[names]
-
 ---[[ end included font-ltx.lua ]]
 
 --[[doc--
@@ -1411,9 +1423,13 @@ local tlig = {
             [0x2014] = {0x002D, 0x002D, 0x002D}, -- emdash
             [0x201C] = {0x2018, 0x2018},         -- quotedblleft
             [0x201D] = {0x2019, 0x2019},         -- quotedblright
-            [0x201E] = {0x002C, 0x002C},         -- quotedblbase
             [0x00A1] = {0x0021, 0x2018},         -- exclamdown
             [0x00BF] = {0x003F, 0x2018},         -- questiondown
+            --- next three originate in T1 encoding; Xetex applies
+            --- them too
+            [0x201E] = {0x002C, 0x002C},         -- quotedblbase
+            [0x00AB] = {0x003C, 0x003C},         -- LEFT-POINTING DOUBLE ANGLE QUOTATION MARK
+            [0x00BB] = {0x003E, 0x003E},         -- RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK
         },
         flags    = { },
     },
@@ -1430,8 +1446,8 @@ local tlig = {
     },
 }
 
-otf.addfeature("tlig", tlig)
-otf.addfeature("trep", { }) -- empty, all in tlig now
+otf.addfeature ("tlig", tlig)
+otf.addfeature ("trep", { })
 
 local anum_arabic = { --- these are the same as in font-otc
     [0x0030] = 0x0660,
