@@ -1,4 +1,4 @@
-# $Id: TLPDB.pm 30347 2013-05-09 18:45:54Z karl $
+# $Id: TLPDB.pm 31361 2013-08-06 01:50:48Z preining $
 # TeXLive::TLPDB.pm - module for using tlpdb files
 # Copyright 2007-2013 Norbert Preining
 # This file is licensed under the GNU General Public License version 2
@@ -6,7 +6,7 @@
 
 package TeXLive::TLPDB;
 
-my $svnrev = '$Revision: 30347 $';
+my $svnrev = '$Revision: 31361 $';
 my $_modulerevision;
 if ($svnrev =~ m/: ([0-9]+) /) {
   $_modulerevision = $1;
@@ -1410,7 +1410,7 @@ sub install_package_files {
       next;
     }
     if ($reloc) {
-      if ($totlpdb->setting("usertree")) {
+      if ($self->setting("usertree")) {
         $tlpobj->cancel_reloc_prefix;
       } else {
         $tlpobj->replace_reloc_prefix;
@@ -1431,7 +1431,7 @@ sub install_package_files {
     # Run the post installation code in the postaction tlpsrc entries
     # in case we are on w32 and the admin did install for himself only
     # we switch off admin mode
-    if (win32() && admin() && !$totlpdb->option("w32_multi_user")) {
+    if (win32() && admin() && !$self->option("w32_multi_user")) {
       non_admin();
     }
     # for now desktop_integration maps to both installation
